# What is Nodedge?

<div>

Nodedge is the first open-source Python editor for scientific and engineering applications. 
It provides a comprehensive graphic environment for 
accelerating the design, simulation, analysis, and deployment of complex scientific and engineering systems.
With Nodedge, you can:
* design and **simulate dynamical systems** such as robotic, physical, and chemical systems
* **generate Python code** from your graphic models with a simple click
* interactively plot and edit the data generated by your simulation
* **plot and analyze log files** with common extensions like csv, mf4, mat, and others.

The user interface is designed for simplicity, allowing you to **sketch your program in a diagram-like flow** 
by drag-and-dropping graphic blocks. A set of built-in block libraries implements common functionalities like scalar and matrix operations, 
integration, differentiation, and many others. You can also implement your own custom blocks, reuse and 
share them with the community. 

Nodedge is **written in Python**, the number one programming language in the world for the number of users, and its code is fully 
transparent and **freely available on GitHub** for community collaboration and contributions. This ensures that Nodedge is 
constantly evolving and improving based on the latest needs of its users.
</div>
<br>

<center><img src="media/concept/nodedge_apps.png" alt="nodedge" width="800" /></center>
<br>


## Physical simulation

The Nodedge simulation app allows you to simulate various types of dynamic systems and processes. 
These simulations can suit a variety of purposes, such as testing and analyzing the behavior of a physical model, 
predicting the outcomes of a chemical reaction, or studying the movements and actions of a robotic system.
To learn more about it, check out the [examples](resources.md#quick-start-examples) provided.

<center><img src="media/simulator_icon.png" alt="nodedge_interface" width="150" /></center>

## Data plotting and analysis

Nodedge provides you with an interactive plotting app, Dats. Within Dats, you can easily import the data generated by 
your simulation, plot, edit, and export it.
You can create new variables, also called signals, by modifying the existing ones, filtering, and combining them.
To learn more about it, check out the [examples](resources.md#quick-start-examples) provided.

<center><img src="media/plotter_icon.png" alt="nodedge_interface" width="150" /></center>

## Code generation

Nodedge supports the generation of standalone Python scripts from any graphical model.
In the near future, Nodedge has the goal to support code generation for various microcontroller targets. 
This means that users will be able to create their models graphically and then use Nodedge to convert those models into 
code that can be understood by off-the-shelf microcontrollers and uploaded to the hardware. 

<center><img src="media/code_generation_icon.png" alt="nodedge_interface" width="150" /></center>

# Industries

Nodedge can provide support to many industries. It accelerates R&D processes from engineering design to system development and deployment. 
Are you curious about how to integrate it into your current workflow or do you need custom support for your application?

[Contact our team](mailto:admin@nodedge.io)!

<div class="row">
  <div class="column">
    <img src="media/industries/robotics.jpg" alt="robotics" style="width:100%">
    Robotics
  </div>
  <div class="column">
    <img src="media/industries/automotive.jpg" alt="automotive" style="width:100%">
    Automotive
  </div>
  <div class="column">
    <img src="media/industries/uavs.jpg" alt="uavs" style="width:100%">
    UAVs
  </div>
  <div class="column">
    <img src="media/industries/clean_energy.jpg" alt="clean_energy" style="width:100%">
    Energy & electrification
  </div>
  <div class="column">
    <img src="media/industries/aerospace.jpg" alt="aerospace" style="width:100%">
    Aerospace
  </div>
  <div class="column">
    <img src="media/industries/pharma.jpg" alt="pharma" style="width:100%">
    Pharmaceutical
  </div>
</div>

